// Design:      Simple RPN calculator
// File:        system.v
// Description: Calculator system for development board implementation.
// Author:      Jorge Juan-Chico <jjchico@dte.us.es>
// Date:        2021-03-01 (initial)

module system (
    input wire clk,             // clock (rising edge)
    input wire enter,           // enter key
    input wire add,             // add key
    input wire sub,             // subtract key
    input wire neg,              // plus/minus key
    input wire [7:0] x_in,      // data input
    output wire f_c,            // flag carry
    output wire f_v,            // flag overflow
    output wire [3:0] an,       // 7-segment anode control
    output wire [0:6] seg,      // 7-segment code (active-low)
    output wire dp              // 7-segment decimal point output
);


	wire reset;
	assign reset = 1;  /*  dado que system no tiene entrada de reset pero la calculadora sí la tiene le hemos asignado valor 1 para que no quede indeterminada */
    wire [7:0] cable_yreg;  /* cable para conectar la salida yreg de calculator con las entradas x1 y x0 de display_ctrl */
    
    //escribir instancia de la calculadora
    
    
      // 7-segment controller instance
    display_ctrl #(.cdbits(18), .hex(1)) display_ctrl (
        .ck(clk),              // system clock
        .x3(x_in[7:4]),        // display digits, from left to right
        .x2(x_in[3:0]),
        .x1(cable_yreg[7:4]),
        .x0(cable_yreg[3:0]),
        .dp_in(4'b1011),     // decimal point vector
        .seg(seg),       // 7-segment output
        .an(an),        // anode output
        .dp(dp)               // decimal point output
    );

endmodule
